/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2004 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package jorm.rdb;

// Import clauses required by jorm
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PClassMappingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionNoDSI;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PMappingCallback;
import org.objectweb.jorm.api.PMappingStructuresManager;
import org.objectweb.jorm.api.PNameIterator;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.genclass.api.FieldDesc;
import org.objectweb.jorm.genclass.api.GenClassProp;
import org.objectweb.jorm.lib.EmptyPNameIterator;
import org.objectweb.jorm.lib.MultiplePNameIterator;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNameManager;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;
//import for the rdb mapper
import org.objectweb.jorm.api.PPrefetchTupleCollection;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;
import org.objectweb.jorm.mapper.rdb.adapter.api.JoinedTable;
import org.objectweb.jorm.mapper.rdb.lib.RdbExtentGenInfos;
import org.objectweb.jorm.mapper.rdb.lib.RdbTupleCollection;
import org.objectweb.jorm.mapper.rdb.lib.PClassMappingDecoder;
import org.objectweb.jorm.mapper.rdb.lib.PMapperRdb;
import org.objectweb.jorm.mapper.rdb.lib.RdbPMappingStructuresManager;
import org.objectweb.jorm.mapper.rdb.lib.RdbConnectionWrapper;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbClassMapping;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbMapping;
import org.objectweb.jorm.mapper.rdb.lib.RdbPPolymorphicClass;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.lib.MedorEvaluator;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.eval.api.ConnectionResources;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.CalculatedParameterOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.query.rdb.lib.BasicRdbStringQueryLeaf;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.lib.SelectProject;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.datasource.lib.ConnectionFactoryDataStore;
import org.objectweb.medor.expression.lib.BasicBinaryArithmeticOperator;
import org.objectweb.medor.expression.lib.Plus;
import org.objectweb.medor.eval.prefetch.api.PrefetchBuffer;
import org.objectweb.medor.eval.prefetch.api.IntermediaryPrefetchBuffer;
import org.objectweb.medor.eval.prefetch.lib.IntermediaryPrefetchBufferImpl;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.jorm.lib.EncodePName;
import org.objectweb.medor.filter.jorm.lib.CompositePName;
import org.objectweb.medor.filter.jorm.lib.SinglePName;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.optim.lib.IndexesGenerator;
import org.objectweb.perseus.persistence.api.ConnectionHolder;
import org.objectweb.perseus.persistence.api.WorkingSet;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Collections;
import java.util.Collection;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.mapper.rdb.genclass.RdbFieldDesc;
import org.objectweb.jorm.mapper.rdb.genclass.RdbGenClassProp;
import org.objectweb.jorm.mapper.rdb.lib.RdbPrefetchablePCM;
// There are Serialized fields
import org.objectweb.jorm.mapper.rdb.lib.RdbBlob;
import java.io.ObjectInputStream;
import java.io.Serializable;


// Import clauses required by the object BnNode
import jorm.BnNodeAccessor;
import jorm.BnNodePKPNG;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

public class BnNodeMapping
	extends org.objectweb.jonas_ejb.container.jorm.RdbFactory
	implements PClassMapping, PClassMappingCtrl, Loggable
		, PClassMappingDecoder
		, RdbPrefetchablePCM
	, PNameGetter, jorm.BnNodePKPNG
    {
	protected static final String MAPPER_NAME = "rdb";
	protected static final String PROJECT_NAME = "jonas";
	//Fields
	protected static final String activation_NAME = "activation";
	protected static final String activityPerformer_NAME = "activityPerformer";
	protected static final String anticipable_NAME = "anticipable";
	protected static final String bnHooks_NAME = "bnHooks";
	protected static final String bnInterHooks_NAME = "bnInterHooks";
	protected static final String bnNodePerformerAssign_NAME = "bnNodePerformerAssign";
	protected static final String bnProject_NAME = "bnProject";
	protected static final String bnProperties_NAME = "bnProperties";
	protected static final String bnRole_NAME = "bnRole";
	protected static final String creationDate_NAME = "creationDate";
	protected static final String creator_NAME = "creator";
	protected static final String deadlines_NAME = "deadlines";
	protected static final String description_NAME = "description";
	protected static final String endDate_NAME = "endDate";
	protected static final String executor_NAME = "executor";
	protected static final String id_NAME = "id";
	protected static final String inBnAgentEdges_NAME = "inBnAgentEdges";
	protected static final String inBnEdges_NAME = "inBnEdges";
	protected static final String name_NAME = "name";
	protected static final String outBnEdges_NAME = "outBnEdges";
	protected static final String relativeDeadlines_NAME = "relativeDeadlines";
	protected static final String startDate_NAME = "startDate";
	protected static final String state_NAME = "state";
	protected static final String transition_NAME = "transition";
	protected static final String type_NAME = "type";
	//Hidden fields
	protected static final String bnNodePerformerAssign_id_NAME = "bnNodePerformerAssign_id";
	protected static final String bnProject_id_NAME = "bnProject_id";
	protected static final String bnRole_id_NAME = "bnRole_id";

	private final static String[][] SUPER_TYPES =  {};

	/** Fully quailied name of the persitent class */
	protected final static String JORM_CLASS_NAME =
		"jorm.BnNode";
	/** The actual binder which does the binding action */
	private PBinder binder;
	private PNameCoder classPNC;

	/** The meta object of the current jorm class */
	private Class moc;
	/** The mapper linked to this PClassMapping */
	private PMapper mapper;
	public PNameCoder bnHooksPnc;
	PClassMapping bnHooksGCM;
	public PNameCoder bnInterHooksPnc;
	PClassMapping bnInterHooksGCM;
	public PNameCoder bnNodePerformerAssignPnc;
	public PNameCoder bnProjectPnc;
	public PNameCoder bnPropertiesPnc;
	PClassMapping bnPropertiesGCM;
	public PNameCoder bnRolePnc;
	public PNameCoder inBnAgentEdgesPnc;
	PClassMapping inBnAgentEdgesGCM;
	public PNameCoder inBnEdgesPnc;
	PClassMapping inBnEdgesGCM;
	public PNameCoder outBnEdgesPnc;
	PClassMapping outBnEdgesGCM;
	/** the PClassMapping of the sub type */
	private PClassMapping[] subPCMs = null;

	//variables for the resolve method
	private int beginIndex = 0;
	private QueryTree queryTree = null;
	private FieldOperand[] fieldOperands = null;
	private String[] compositeFieldNames = null;
 	/**
	 * The logger instance used by this XMapping instance
	 * and the XBinding instances
	 */
	public Logger logger;
	public LoggerFactory loggerFactory;

	public BnNodeMapping() {
 	}
	public PNameCoder getBnHooksPNameCoder() {
	    return bnHooksPnc;
	}
	public PNameCoder getBnInterHooksPNameCoder() {
	    return bnInterHooksPnc;
	}
	public PNameCoder getBnNodePerformerAssignPNameCoder() {
	    return bnNodePerformerAssignPnc;
	}
	public PNameCoder getBnProjectPNameCoder() {
	    return bnProjectPnc;
	}
	public PNameCoder getBnPropertiesPNameCoder() {
	    return bnPropertiesPnc;
	}
	public PNameCoder getBnRolePNameCoder() {
	    return bnRolePnc;
	}
	public PNameCoder getInBnAgentEdgesPNameCoder() {
	    return inBnAgentEdgesPnc;
	}
	public PNameCoder getInBnEdgesPNameCoder() {
	    return inBnEdgesPnc;
	}
	public PNameCoder getOutBnEdgesPNameCoder() {
	    return outBnEdgesPnc;
	}

	// IMPLEMENTATION OF METHODS FROM THE PClassMappingCtrl INTERFACE

    public void setPNameCoder(PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setPNameCoder(String fn, PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		if (bnHooks_NAME.equals(fn)) {
			bnHooksPnc = (PNamingContext) pnc;
			return;
		}
		if (bnInterHooks_NAME.equals(fn)) {
			bnInterHooksPnc = (PNamingContext) pnc;
			return;
		}
		if (bnNodePerformerAssign_NAME.equals(fn)) {
			bnNodePerformerAssignPnc = (PNamingContext) pnc;
			return;
		}
		if (bnProject_NAME.equals(fn)) {
			bnProjectPnc = (PNamingContext) pnc;
			return;
		}
		if (bnProperties_NAME.equals(fn)) {
			bnPropertiesPnc = (PNamingContext) pnc;
			return;
		}
		if (bnRole_NAME.equals(fn)) {
			bnRolePnc = (PNamingContext) pnc;
			return;
		}
		if (inBnAgentEdges_NAME.equals(fn)) {
			inBnAgentEdgesPnc = (PNamingContext) pnc;
			return;
		}
		if (inBnEdges_NAME.equals(fn)) {
			inBnEdgesPnc = (PNamingContext) pnc;
			return;
		}
		if (outBnEdges_NAME.equals(fn)) {
			outBnEdgesPnc = (PNamingContext) pnc;
			return;
		}
		throw new UnsupportedOperationException("This class does'nt contain field reference: " + fn);
    }

    public void setGenClassMapping(PClassMapping pcm)
            throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setGenClassMapping(String fn, PClassMapping pcm)
            throws UnsupportedOperationException {
		if (bnHooks_NAME.equals(fn)) {
			bnHooksGCM = pcm;
			return;
		}
		if (bnInterHooks_NAME.equals(fn)) {
			bnInterHooksGCM = pcm;
			return;
		}
		if (bnProperties_NAME.equals(fn)) {
			bnPropertiesGCM = pcm;
			return;
		}
		if (inBnAgentEdges_NAME.equals(fn)) {
			inBnAgentEdgesGCM = pcm;
			return;
		}
		if (inBnEdges_NAME.equals(fn)) {
			inBnEdgesGCM = pcm;
			return;
		}
		if (outBnEdges_NAME.equals(fn)) {
			outBnEdgesGCM = pcm;
			return;
		}
		throw new UnsupportedOperationException("This class does not contain a " + fn + " field referencing a generic class");
    }

    public void setProjectName(String projectname) {
    }


	// IMPLEMENTATION OF METHODS FROM THE PClassMapping INTERFACE

	public void init(PMappingCallback m, MetaObject metaclass) throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		mapper = m;
		moc = (Class) metaclass;
		//the getPType method inits the ptype if not already done
		PType ptype = mapper.getPTypeSpace().createPType(
				"jorm.BnNode",
				SUPER_TYPES);
		setPType(ptype);
        // PType Assignement to the PNC managing the reference bnProject
        String[][] bnProjectICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            "jorm.BnProject",
            bnProjectICDef);
        bnProjectPnc.setPType(ptype);
        
        // PType Assignement to the PNC managing the reference bnRole
        String[][] bnRoleICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            "jorm.BnRole",
            bnRoleICDef);
        bnRolePnc.setPType(ptype);
        
        // PType Assignement to the PNC managing the reference bnNodePerformerAssign
        String[][] bnNodePerformerAssignICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            "jorm.BnNodePerformerAssign",
            bnNodePerformerAssignICDef);
        bnNodePerformerAssignPnc.setPType(ptype);
        
         try {
			adapter = ((PMapperRdb) mapper).getRdbAdapter();
			if (adapter == null) {
				throw new PException("No rdb adapter found on the mapper: " + mapper);
			}

         // Initialization of the gen class /jorm.BnNode/inBnEdges
        FieldDesc[] fds = null;
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnNode_id", "id", PTypeSpace.STRING, "BnNode_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) inBnEdgesGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) inBnEdgesGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnEdge_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) inBnEdgesGCM).setElemDesc(fds);
			((RdbGenClassProp) inBnEdgesGCM).setTableName("BnEdge");
			((RdbGenClassProp) inBnEdgesGCM).setColocated(true);
			((RdbGenClassProp) inBnEdgesGCM).setColocatedMaster(false);
			((RdbGenClassProp) inBnEdgesGCM).setReadOnly(true);
			((RdbGenClassProp) inBnEdgesGCM).setTypeConverter(adapter);
        String[] inBnEdgesGCDef = {"java.util.Collection"};
        String[][] inBnEdgesICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            inBnEdgesGCDef,
            "jorm.BnEdge",
            inBnEdgesICDef);
        ((GenClassProp) inBnEdgesGCM).setPType(ptype);
        inBnEdgesPnc.setPType(ptype);
        inBnEdgesGCM.init(m, null);

         // Initialization of the gen class /jorm.BnNode/outBnEdges
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnNode_id", "id", PTypeSpace.STRING, "fk_innode", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) outBnEdgesGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) outBnEdgesGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnEdge_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) outBnEdgesGCM).setElemDesc(fds);
			((RdbGenClassProp) outBnEdgesGCM).setTableName("BnEdge");
			((RdbGenClassProp) outBnEdgesGCM).setColocated(true);
			((RdbGenClassProp) outBnEdgesGCM).setColocatedMaster(false);
			((RdbGenClassProp) outBnEdgesGCM).setReadOnly(true);
			((RdbGenClassProp) outBnEdgesGCM).setTypeConverter(adapter);
        String[] outBnEdgesGCDef = {"java.util.Collection"};
        String[][] outBnEdgesICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            outBnEdgesGCDef,
            "jorm.BnEdge",
            outBnEdgesICDef);
        ((GenClassProp) outBnEdgesGCM).setPType(ptype);
        outBnEdgesPnc.setPType(ptype);
        outBnEdgesGCM.init(m, null);

         // Initialization of the gen class /jorm.BnNode/bnInterHooks
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnNode_id", "id", PTypeSpace.STRING, "BnNode_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnInterHooksGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnInterHooksGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnNodeInterHook_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnInterHooksGCM).setElemDesc(fds);
			((RdbGenClassProp) bnInterHooksGCM).setTableName("BnNodeInterHook");
			((RdbGenClassProp) bnInterHooksGCM).setColocated(true);
			((RdbGenClassProp) bnInterHooksGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnInterHooksGCM).setReadOnly(true);
			((RdbGenClassProp) bnInterHooksGCM).setTypeConverter(adapter);
        String[] bnInterHooksGCDef = {"java.util.Collection"};
        String[][] bnInterHooksICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnInterHooksGCDef,
            "jorm.BnNodeInterHook",
            bnInterHooksICDef);
        ((GenClassProp) bnInterHooksGCM).setPType(ptype);
        bnInterHooksPnc.setPType(ptype);
        bnInterHooksGCM.init(m, null);

         // Initialization of the gen class /jorm.BnNode/bnHooks
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnNode_id", "id", PTypeSpace.STRING, "BnNode_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnHooksGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnHooksGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnNodeHook_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnHooksGCM).setElemDesc(fds);
			((RdbGenClassProp) bnHooksGCM).setTableName("BnNodeHook");
			((RdbGenClassProp) bnHooksGCM).setColocated(true);
			((RdbGenClassProp) bnHooksGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnHooksGCM).setReadOnly(true);
			((RdbGenClassProp) bnHooksGCM).setTypeConverter(adapter);
        String[] bnHooksGCDef = {"java.util.Collection"};
        String[][] bnHooksICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnHooksGCDef,
            "jorm.BnNodeHook",
            bnHooksICDef);
        ((GenClassProp) bnHooksGCM).setPType(ptype);
        bnHooksPnc.setPType(ptype);
        bnHooksGCM.init(m, null);

         // Initialization of the gen class /jorm.BnNode/inBnAgentEdges
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnNode_id", "id", PTypeSpace.STRING, "BnNode_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) inBnAgentEdgesGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) inBnAgentEdgesGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnAgentEdge_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) inBnAgentEdgesGCM).setElemDesc(fds);
			((RdbGenClassProp) inBnAgentEdgesGCM).setTableName("BnAgentEdge");
			((RdbGenClassProp) inBnAgentEdgesGCM).setColocated(true);
			((RdbGenClassProp) inBnAgentEdgesGCM).setColocatedMaster(false);
			((RdbGenClassProp) inBnAgentEdgesGCM).setReadOnly(true);
			((RdbGenClassProp) inBnAgentEdgesGCM).setTypeConverter(adapter);
        String[] inBnAgentEdgesGCDef = {"java.util.Collection"};
        String[][] inBnAgentEdgesICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            inBnAgentEdgesGCDef,
            "jorm.BnAgentEdge",
            inBnAgentEdgesICDef);
        ((GenClassProp) inBnAgentEdgesGCM).setPType(ptype);
        inBnAgentEdgesPnc.setPType(ptype);
        inBnAgentEdgesGCM.init(m, null);

         // Initialization of the gen class /jorm.BnNode/bnProperties
		fds = new FieldDesc[1];
			fds[0] = new RdbFieldDesc("jorm.BnNode_id", "id", PTypeSpace.STRING, "BnNode_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
		((GenClassProp) bnPropertiesGCM ).setIdentifierDesc(fds);
		fds = new FieldDesc[0];
 		((GenClassProp) bnPropertiesGCM ).setIndexDesc(fds);
		fds = new FieldDesc[2];
			fds[1] = new RdbFieldDesc("jorm.BnNodeProperty_id", "id", PTypeSpace.STRING, "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE));
			fds[0] = new RdbFieldDesc(null, null, PTypeSpace.REFTOP, null, null);
 		((GenClassProp) bnPropertiesGCM).setElemDesc(fds);
			((RdbGenClassProp) bnPropertiesGCM).setTableName("BnNodeProperty");
			((RdbGenClassProp) bnPropertiesGCM).setColocated(true);
			((RdbGenClassProp) bnPropertiesGCM).setColocatedMaster(false);
			((RdbGenClassProp) bnPropertiesGCM).setReadOnly(true);
			((RdbGenClassProp) bnPropertiesGCM).setTypeConverter(adapter);
        String[] bnPropertiesGCDef = {"java.util.Collection"};
        String[][] bnPropertiesICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            bnPropertiesGCDef,
            "jorm.BnNodeProperty",
            bnPropertiesICDef);
        ((GenClassProp) bnPropertiesGCM).setPType(ptype);
        bnPropertiesPnc.setPType(ptype);
        bnPropertiesGCM.init(m, null);
 		//register the structures and dependencies
		init(mapper.getPMappingStructuresManager());
        } catch (Exception e) {
			if(e instanceof RdbAdapterException)
				throw new PException(e, "Column type computing error: ");
			else
				e.printStackTrace();
		} 
			computeReadQuery();
		try {
			computeExtentQuery();
		} catch (ExpressionException e) {
			throw new PException(e);
		}
		computePrefetchedExtentQuery();
		}

	public void init(PMappingStructuresManager pmsm) throws PException {
		pmsm.declareClass(JORM_CLASS_NAME);
		try {
        RdbPMappingStructuresManager msm = (RdbPMappingStructuresManager) pmsm;
        if (mapper == null) {
	       	mapper = msm.getPMapper();
	    }
	    if (adapter == null) {
			adapter = ((PMapperRdb) mapper).getRdbAdapter();
			if (adapter == null) {
				throw new PException("No rdb adapter found on the mapper: " + mapper);
			}
		}
	    // Initialization of the structures of the class jorm.BnNode
			        	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "startdate", adapter.getSqlType(PTypeSpace.DATE.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "executor", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "anticipable", adapter.getSqlType(PTypeSpace.BOOLEAN.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "enddate", adapter.getSqlType(PTypeSpace.DATE.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "type", adapter.getSqlType(PTypeSpace.INT.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "transition", adapter.getSqlType(PTypeSpace.SERIALIZED.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "activityPerformer", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "deadlines", adapter.getSqlType(PTypeSpace.SERIALIZED.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "relativedeadlines", adapter.getSqlType(PTypeSpace.SERIALIZED.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "BnProject_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, -1, -1), true, true, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "BnRole_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "creator", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "description", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "state", adapter.getSqlType(PTypeSpace.INT.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "activation", adapter.getSqlType(PTypeSpace.SERIALIZED.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "name", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNode", "creationdate", adapter.getSqlType(PTypeSpace.DATE.getTypeCode(), false, -1, -1), false, false, true);
	    			        	msm.addTableColumn(JORM_CLASS_NAME, "BnNodePerformerAssign", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, false);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnNodePerformerAssign", "BnNode_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), true, false, false);
	    		        // Initialization of the structures of the gen class /jorm.BnNode/inBnEdges
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnEdge", "BnNode_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnEdge", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnNode/outBnEdges
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnEdge", "fk_innode", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnEdge", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnNode/bnInterHooks
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnNodeInterHook", "BnNode_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnNodeInterHook", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnNode/bnHooks
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnNodeHook", "BnNode_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnNodeHook", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnNode/inBnAgentEdges
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnAgentEdge", "BnNode_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnAgentEdge", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
			        // Initialization of the structures of the gen class /jorm.BnNode/bnProperties
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnNodeProperty", "BnNode_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
		        	msm.addTableColumn(JORM_CLASS_NAME, "BnNodeProperty", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, RdbAdapter.NOSIZE, RdbAdapter.NOSIZE), false, false, false);
						//dependencies
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnAgentEdge");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnNodePerformerAssign");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnNodeHook");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnEdge");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnNodeProperty");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnRole");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnProject");
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnNodeInterHook");
		} catch(Exception e) {
			throw new PException("Error while registering mapping structures: " + e.getMessage());
		}
		classDefined(pmsm);
	}
	
	protected void classDefined() throws PException {
		//register class in the cluster
        classDefined(mapper.getPMappingStructuresManager());
	}

	public void classDefined(PMappingStructuresManager pmsm) throws PException {
		if (pmsm == null) {
			throw new PException("PMappingStructureManager is null: " + JORM_CLASS_NAME + " cannot be registered in a cluster.");
		}
		//register class in the cluster
        pmsm.classDefined(JORM_CLASS_NAME);
	}
	
	public PMapper getPMapper() {
		return mapper;
	}

	public PBinder getPBinder() {
		return binder;
	}

	public PNameCoder getClassPNameCoder() {
		return classPNC;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains references*/
	public PNameCoder getPNameCoder()
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PNameCoder getPNameCoder(String fn)
		throws UnsupportedOperationException {
			if (bnHooks_NAME.equals(fn))
				return bnHooksPnc;
			if (bnInterHooks_NAME.equals(fn))
				return bnInterHooksPnc;
			if (bnNodePerformerAssign_NAME.equals(fn))
				return bnNodePerformerAssignPnc;
			if (bnProject_NAME.equals(fn))
				return bnProjectPnc;
			if (bnProperties_NAME.equals(fn))
				return bnPropertiesPnc;
			if (bnRole_NAME.equals(fn))
				return bnRolePnc;
			if (inBnAgentEdges_NAME.equals(fn))
				return inBnAgentEdgesPnc;
			if (inBnEdges_NAME.equals(fn))
				return inBnEdgesPnc;
			if (outBnEdges_NAME.equals(fn))
				return outBnEdgesPnc;
		throw new UnsupportedOperationException("This class does'nt contain field reference " + fn);
	}

	public String getClassName() {
		return JORM_CLASS_NAME;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains another gen class*/
	public PClassMapping getGenClassMapping() throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PClassMapping getGenClassMapping(String fn)
		throws UnsupportedOperationException {
			if (bnHooks_NAME.equals(fn))
				return bnHooksGCM;
			if (bnInterHooks_NAME.equals(fn))
				return bnInterHooksGCM;
			if (bnProperties_NAME.equals(fn))
				return bnPropertiesGCM;
			if (inBnAgentEdges_NAME.equals(fn))
				return inBnAgentEdgesGCM;
			if (inBnEdges_NAME.equals(fn))
				return inBnEdgesGCM;
			if (outBnEdges_NAME.equals(fn))
				return outBnEdgesGCM;
			throw new UnsupportedOperationException("This class does not contain a " + fn + " field which would reference a generic class");
	}

	public boolean isConform(String mapperName) {
		return mapperName.startsWith(MAPPER_NAME);
	}

	public String getProjectName() {
		return PROJECT_NAME;
	}

	public MetaObject getMetaInfo() {
		return moc;
	}

	public String[][] getSuperClassNames() {
		return SUPER_TYPES;
	}

	public void setPBinder(PBinder b) {
		binder = b;
	}

	public void setClassPNameCoder(PNameCoder pnc) throws PException {
		classPNC = pnc;
	}

	public PBinding createPBinding() throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		PBinding binding = new jorm.BnNodeBinding();
		binding.init(this);
		return binding;
	}
	/** @deprecated */
    public Iterator getPNameIterator(Object conn) throws PException {
		return getPNameIterator(conn, false, false, null);
    }

	public PNameIterator getPNameIterator(Object conn,
	                                      boolean withSubType,
	                                      boolean prefetching,
	                                      Object txctx) throws PException {
		PNameIterator it = null;
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String mapperName = mapper.getMapperName();
			if(mapperName.indexOf(".") != -1)
				mapperName = mapperName.substring(0, mapperName.indexOf("."));	
			RdbMapping rdbm = (RdbMapping) mapper.getMetaInfoManager().getClass(this.getClassName()).getClassProject(this.getProjectName()).getMapping(mapperName);
			RdbClassMapping rdbcm = (RdbClassMapping) rdbm.getClassMapping();
			String queryToExecute = "";
			int beginIndex = 0;
			if(withSubType){
				//the list of fields belonging to the primary key
				String pk = "BnNode.id";
				queryToExecute = rdbcm.getRdbInheritanceQuery().getExtentQuery(mapper,prefetching, true, pk, this);	
				//System.out.println("query to execute:" + queryToExecute + " compared with the generated query: " + (prefetching ? prefetchedExtentQuery : extentQuery));
				beginIndex =  rdbcm.getRdbInheritanceQuery().getExtentFieldsNames(prefetching, true).size() + 1;
			}
			else{
				queryToExecute = (prefetching ? prefetchedExtentQuery : extentQuery);
				//System.out.println("query to execute:" + queryToExecute);
				//compute the begining index for the PName: number of fields to select + 1
				beginIndex = 19 + 1;
			}
			PreparedStatement ps = connection.prepareStatement(queryToExecute);
			it = new BnNodeResultIterator(txctx, ps, ps.executeQuery(), binder, prefetching, withSubType, logger, beginIndex, this);
        } catch (SQLException e) {
            throw new PExceptionIO(e, "SQL problem while computing the PName Iterator: class <BnNode>.");
        } catch (MedorException e) {
            throw new PExceptionIO(e, "Medor problem while initializing the PName Iterator: class <BnNode>.");
        }
		return it;
	}

	public PClassMapping[] getSubPCMs() throws PException {
		PType ptype = getPType();
		if (ptype == null) {
			throw new PException("No PType defined for the PClassMapping of the class" + JORM_CLASS_NAME);
		}
		PType[] subTypes = ptype.getSubTypes();
		if (subTypes == null || subTypes.length == 0) {
			//No direct subType are mapped then return null
			return null;
		}
		if (subPCMs == null || subPCMs.length != subTypes.length) {
			//compute the PCM array
			subPCMs = new PClassMapping[subTypes.length];
			for (int i=0; i<subTypes.length; i++) {
				subPCMs[i] = mapper.lookup(subTypes[i].getJormName());
			}
		}
		return subPCMs;
	}
	
	// ADD YO //
	public synchronized PName resolve(Object conn, PName pname) throws PException {
				//mapper rdb but not polymorphic
		return pname;
	 	}
	
	public boolean match(Object obj, boolean intermediaryTuple) throws PException {
					return false;
				}
	
	public PName getDecodedPName(TupleCollection tc, PName pname, boolean intermediaryTuple) throws PException {
		PBinder binder = null;
	//test if the tuple is an instance of this class
	if (this.match(tc, intermediaryTuple)) {
		binder  = this.getPBinder();
	} else {
		//test for all the children
		boolean match = false;
		for (int i = 0; i<subPCMs.length && !match; i++) {
				match = subPCMs[i].match(tc, intermediaryTuple);
				binder = subPCMs[i].getPBinder();
		}
		//if no pcm found, error
		if (!match)
			throw new PException("Problem with Medor while resolving pname on the PCM " + JORM_CLASS_NAME + ". No class found for the pname " + pname.toString() + ".");
	}
			return binder.decodeAbstract(pname.encodeAbstract(), null);
		}
	// END ADD YO //
	
    public void configureRefFields(ReferenceConfigurator rc) throws PException, UnsupportedOperationException {
        PClassMapping gcm = null;
        bnHooksPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnHooks_NAME, new String[]{"java.util.Collection"});
        bnHooksGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnHooks_NAME, new String[]{"java.util.Collection"}, "jorm.BnNodeHook");
        gcm = bnHooksGCM;
        bnInterHooksPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnInterHooks_NAME, new String[]{"java.util.Collection"});
        bnInterHooksGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnInterHooks_NAME, new String[]{"java.util.Collection"}, "jorm.BnNodeInterHook");
        gcm = bnInterHooksGCM;
		bnNodePerformerAssignPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnNodePerformerAssign_NAME, "jorm.BnNodePerformerAssign");
		bnProjectPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnProject_NAME, "jorm.BnProject");
        bnPropertiesPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnProperties_NAME, new String[]{"java.util.Collection"});
        bnPropertiesGCM = rc.getGenClassMapping(JORM_CLASS_NAME, bnProperties_NAME, new String[]{"java.util.Collection"}, "jorm.BnNodeProperty");
        gcm = bnPropertiesGCM;
		bnRolePnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnRole_NAME, "jorm.BnRole");
        inBnAgentEdgesPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, inBnAgentEdges_NAME, new String[]{"java.util.Collection"});
        inBnAgentEdgesGCM = rc.getGenClassMapping(JORM_CLASS_NAME, inBnAgentEdges_NAME, new String[]{"java.util.Collection"}, "jorm.BnAgentEdge");
        gcm = inBnAgentEdgesGCM;
        inBnEdgesPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, inBnEdges_NAME, new String[]{"java.util.Collection"});
        inBnEdgesGCM = rc.getGenClassMapping(JORM_CLASS_NAME, inBnEdges_NAME, new String[]{"java.util.Collection"}, "jorm.BnEdge");
        gcm = inBnEdgesGCM;
        outBnEdgesPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, outBnEdges_NAME, new String[]{"java.util.Collection"});
        outBnEdgesGCM = rc.getGenClassMapping(JORM_CLASS_NAME, outBnEdges_NAME, new String[]{"java.util.Collection"}, "jorm.BnEdge");
        gcm = outBnEdgesGCM;
    }

	public PType getPType() {
		return  binder.getPType();
	}

	public void setPType(PType pt) {
		binder.setPType(pt);
	}


	public void write(PBinding pb, Object conn, PAccessor pa)
		throws PException {
		BnNodeAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof BnNodeAccessor)
			paccessor = (BnNodeAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte status = pb.getStatus();
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_WRITE);
		if (nextState == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized WRITE operation, status=" + status);
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String sqlquery;
		if (status == PBinding.LIFECYCLE_DELTOWRITE) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Remove");
//Table BnNode : readOnly=false mustGenerateUpdate=true colocatedTable=false joins=$tab.joins
                sqlquery = deleteBnNodeTable(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [BnNode]: problematic SQL query -> {" + sqlquery + "}");
                }
//Table BnNodePerformerAssign : readOnly=true mustGenerateUpdate=true colocatedTable=true joins=[org.objectweb.jorm.mapper.rdb.generator.RdbGenJoin@a0e220]
		} else if (status==PBinding.LIFECYCLE_NEWTOWRITE) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Insert");
                sqlquery = insertBnNodeTable(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [BnNode]: problematic SQL query -> {" + sqlquery + "}");
                }
		} else if (status==PBinding.LIFECYCLE_ACTIVEFORIO) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Update");
                sqlquery = updateBnNodeTable(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [BnNode]: problematic SQL query -> {" + sqlquery + "}");
                }
		}
		} catch (java.sql.SQLException se) {
			throw new PExceptionIO(se, "SQL problem while writing the DSI.");
		} catch (java.io.IOException ioe) {
			throw new PExceptionIO(ioe, "Stream IO problem while writing the DSI.");
		} catch (RdbAdapterException rae) {
			throw new PExceptionIO(rae, "SQL problem while writing the DSI.");
		}
		// Changing the current state
		((PBindingCtrl) pb).setStatus(nextState);
	}
	public void read(PBinding pb, Object conn, PAccessor pa) throws PException {
		read(pb, conn, pa, false);
	}
	
	public void read(PBinding pb, Object conn, PAccessor pa, boolean forUpdate) throws PException {
		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Read");
		byte status = pb.getStatus();
		BnNodeAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof BnNodeAccessor)
			paccessor = (BnNodeAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
		PName pname = pb.getPName();
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet resultSet = null;
        PException resex = null;
		try {
            String jormrq = readQuery;
            if (forUpdate) {
            	jormrq = jormrq + " FOR UPDATE";
            }
            pstmt = connection.prepareStatement(jormrq);
            Object _pngId = pname.encodeAbstract();
            if (_pngId instanceof jorm.BnNodePKPNG) {
                adapter.setString(pstmt, 1, ((jorm.BnNodePKPNG) _pngId).pnGetId(null));
            } else if (_pngId instanceof PNameGetter) {
                adapter.setString(pstmt, 1, ((PNameGetter) _pngId).pngetStringField("id", null));
            } else {
                throw new PExceptionNaming("Invalid abstract encoded value for ID PName: " + _pngId);
            }
            resultSet = pstmt.executeQuery();
            if(resultSet.next() == false) {
                throw new PExceptionNoDSI(
                    "No entry matching with the following identifier: " 
                    + pname + ", query=" + jormrq);
            }
            PName refname;
            Object fieldval;
            long notnullval;
			if (paccessor != null) {
				//read public primitive fields
                paccessor.paSetActivation(adapter.getSerialized(resultSet, 3, null));
                paccessor.paSetActivityPerformer(adapter.getString(resultSet, 4, null));
                paccessor.paSetAnticipable(adapter.getBoolean(resultSet, 5, false));
                paccessor.paSetCreationDate(adapter.getDate(resultSet, 6, null, null));
                paccessor.paSetCreator(adapter.getString(resultSet, 7, null));
                paccessor.paSetDeadlines(adapter.getSerialized(resultSet, 8, null));
                paccessor.paSetDescription(adapter.getString(resultSet, 9, null));
                paccessor.paSetEndDate(adapter.getDate(resultSet, 10, null, null));
                paccessor.paSetExecutor(adapter.getString(resultSet, 11, null));
                paccessor.paSetId(adapter.getString(resultSet, 12, null));
                paccessor.paSetName(adapter.getString(resultSet, 14, null));
                paccessor.paSetRelativeDeadlines(adapter.getSerialized(resultSet, 15, null));
                paccessor.paSetStartDate(adapter.getDate(resultSet, 16, null, null));
                paccessor.paSetState(adapter.getInt(resultSet, 17, ((int) -1)));
                paccessor.paSetTransition(adapter.getSerialized(resultSet, 18, null));
                paccessor.paSetType(adapter.getInt(resultSet, 19, ((int) -1)));
				//decode PName for references
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@5db088 / cnId=9 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@f0cda1} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@2fbdd2 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@6ed322 / nd.nr.cn={id=bnNodePerformerAssign_id} / fieldName=bnNodePerformerAssign / join=org.objectweb.jorm.mapper.rdb.generator.RdbGenJoin@a0e220
                refname = bnNodePerformerAssignPnc.decodeAbstract(getInstancePNG9(), resultSet);
                paccessor.paSetBnNodePerformerAssign(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@14c0cb0 / cnId=1 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1d529be} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@162a60a / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@118d189 / nd.nr.cn={id=id} / fieldName=bnInterHooks / join=null
                refname = bnInterHooksPnc.decodeAbstract(getInstancePNG1(), resultSet);
                paccessor.paSetBnInterHooks(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1a78424 / cnId=4 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1d529be} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@a450bb / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@118d189 / nd.nr.cn={id=id} / fieldName=bnProperties / join=null
                refname = bnPropertiesPnc.decodeAbstract(getInstancePNG4(), resultSet);
                paccessor.paSetBnProperties(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@d48785 / cnId=8 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1d529be} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@1da692f / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@118d189 / nd.nr.cn={id=id} / fieldName=inBnEdges / join=null
                refname = inBnEdgesPnc.decodeAbstract(getInstancePNG8(), resultSet);
                paccessor.paSetInBnEdges(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@10d3d99 / cnId=5 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@473996} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@1b04310 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@81a197 / nd.nr.cn={id=bnProject_id} / fieldName=bnProject / join=null
                refname = bnProjectPnc.decodeAbstract(getInstancePNG5(), resultSet);
                paccessor.paSetBnProject(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@15bd8f / cnId=3 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1d529be} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@578fd6 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@118d189 / nd.nr.cn={id=id} / fieldName=outBnEdges / join=null
                refname = outBnEdgesPnc.decodeAbstract(getInstancePNG3(), resultSet);
                paccessor.paSetOutBnEdges(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@15034e0 / cnId=7 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1d529be} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@182752b / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@118d189 / nd.nr.cn={id=id} / fieldName=bnHooks / join=null
                refname = bnHooksPnc.decodeAbstract(getInstancePNG7(), resultSet);
                paccessor.paSetBnHooks(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@bf8cf5 / cnId=6 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1d529be} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@1030dda / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@118d189 / nd.nr.cn={id=id} / fieldName=inBnAgentEdges / join=null
                refname = inBnAgentEdgesPnc.decodeAbstract(getInstancePNG6(), resultSet);
                paccessor.paSetInBnAgentEdges(refname, conn);
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1bf61a0 / cnId=2 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a28182} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@182f3da / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@716cb7 / nd.nr.cn={id=bnRole_id} / fieldName=bnRole / join=null
                refname = bnRolePnc.decodeAbstract(getInstancePNG2(), resultSet);
                paccessor.paSetBnRole(refname, conn);
			} else if (paccessorGen != null) {
				//read public primitive fields
                paccessorGen.paSetSerializedField(activation_NAME, adapter.getSerialized(resultSet, 3, null));
                paccessorGen.paSetStringField(activityPerformer_NAME, adapter.getString(resultSet, 4, null));
                paccessorGen.paSetBooleanField(anticipable_NAME, adapter.getBoolean(resultSet, 5, false));
                paccessorGen.paSetDateField(creationDate_NAME, adapter.getDate(resultSet, 6, null, null));
                paccessorGen.paSetStringField(creator_NAME, adapter.getString(resultSet, 7, null));
                paccessorGen.paSetSerializedField(deadlines_NAME, adapter.getSerialized(resultSet, 8, null));
                paccessorGen.paSetStringField(description_NAME, adapter.getString(resultSet, 9, null));
                paccessorGen.paSetDateField(endDate_NAME, adapter.getDate(resultSet, 10, null, null));
                paccessorGen.paSetStringField(executor_NAME, adapter.getString(resultSet, 11, null));
                paccessorGen.paSetStringField(id_NAME, adapter.getString(resultSet, 12, null));
                paccessorGen.paSetStringField(name_NAME, adapter.getString(resultSet, 14, null));
                paccessorGen.paSetSerializedField(relativeDeadlines_NAME, adapter.getSerialized(resultSet, 15, null));
                paccessorGen.paSetDateField(startDate_NAME, adapter.getDate(resultSet, 16, null, null));
                paccessorGen.paSetIntField(state_NAME, adapter.getInt(resultSet, 17, ((int) -1)));
                paccessorGen.paSetSerializedField(transition_NAME, adapter.getSerialized(resultSet, 18, null));
                paccessorGen.paSetIntField(type_NAME, adapter.getInt(resultSet, 19, ((int) -1)));
				//decode PName for references
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1bf61a0 / cnId=2 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a28182} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@182f3da / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@716cb7 / nd.nr.cn={id=bnRole_id} / fieldName=bnRole / join=null
                refname = bnRolePnc.decodeAbstract(getInstancePNG2(), resultSet);
                paccessorGen.paSetRefField(bnRole_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1bf61a0 / cnId=2 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a28182} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@182f3da / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@716cb7 / nd.nr.cn={id=bnRole_id} / fieldName=bnRole / join=null
                refname = bnRolePnc.decodeAbstract(getInstancePNG2(), resultSet);
                paccessorGen.paSetRefField(bnRole_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1bf61a0 / cnId=2 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a28182} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@182f3da / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@716cb7 / nd.nr.cn={id=bnRole_id} / fieldName=bnRole / join=null
                refname = bnRolePnc.decodeAbstract(getInstancePNG2(), resultSet);
                paccessorGen.paSetRefField(bnRole_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1bf61a0 / cnId=2 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a28182} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@182f3da / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@716cb7 / nd.nr.cn={id=bnRole_id} / fieldName=bnRole / join=null
                refname = bnRolePnc.decodeAbstract(getInstancePNG2(), resultSet);
                paccessorGen.paSetRefField(bnRole_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1bf61a0 / cnId=2 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a28182} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@182f3da / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@716cb7 / nd.nr.cn={id=bnRole_id} / fieldName=bnRole / join=null
                refname = bnRolePnc.decodeAbstract(getInstancePNG2(), resultSet);
                paccessorGen.paSetRefField(bnRole_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1bf61a0 / cnId=2 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a28182} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@182f3da / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@716cb7 / nd.nr.cn={id=bnRole_id} / fieldName=bnRole / join=null
                refname = bnRolePnc.decodeAbstract(getInstancePNG2(), resultSet);
                paccessorGen.paSetRefField(bnRole_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1bf61a0 / cnId=2 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a28182} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@182f3da / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@716cb7 / nd.nr.cn={id=bnRole_id} / fieldName=bnRole / join=null
                refname = bnRolePnc.decodeAbstract(getInstancePNG2(), resultSet);
                paccessorGen.paSetRefField(bnRole_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1bf61a0 / cnId=2 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a28182} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@182f3da / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@716cb7 / nd.nr.cn={id=bnRole_id} / fieldName=bnRole / join=null
                refname = bnRolePnc.decodeAbstract(getInstancePNG2(), resultSet);
                paccessorGen.paSetRefField(bnRole_NAME, refname, conn);
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@1bf61a0 / cnId=2 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@1a28182} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@182f3da / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@716cb7 / nd.nr.cn={id=bnRole_id} / fieldName=bnRole / join=null
                refname = bnRolePnc.decodeAbstract(getInstancePNG2(), resultSet);
                paccessorGen.paSetRefField(bnRole_NAME, refname, conn);
			}
		} catch (PException pe) {
			resex = pe;
		} catch (Exception pe) {
			resex = new PExceptionIO(pe, "Problem while reading the DSI.");
		} finally {
		    try {
		        if (resultSet != null) {
			        resultSet.close();
			        resultSet = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
			    	resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		((PBindingCtrl) pb).setStatus(nextState);
	}
	public void read(PBinding pb, Object conn, PAccessor pa, Object txctx) throws PException {
		read(pb, conn, pa, txctx, false);
	}

	public void read(PBinding pb, Object conn, PAccessor pa, Object txctx, boolean forUpdate) throws PException {
	    PrefetchCache pc = getPMapper().getPrefetchCache();
	    if (pc == null || txctx == null) {
	        read(pb, conn, pa, forUpdate);
	        return;
	    }
		PName pname = pb.getPName();
	    try {
	        Tuple tuple = pc.getPrefetchTuple(pname, this, txctx);
	        if (tuple == null) {
	            read(pb, conn, pa, forUpdate);
	            return;
	        }
		    BnNodeAccessor paccessor = null;
		    PAccessorGen paccessorGen = null;
		    if (pa == null)
			    throw new PExceptionProtocol("Accessor must not be null");
		    else if (pa instanceof BnNodeAccessor)
			    paccessor = (BnNodeAccessor) pa;
		    else if (pa instanceof PAccessorGen)
			    paccessorGen = (PAccessorGen) pa;
		    else
			    throw new PExceptionProtocol("Wrong type of accessor: " + pa);
			byte status = pb.getStatus();
		    byte nextState = PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		    if (nextState==PBinding.LIFECYCLE_ERROR)
			    throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
            Object fval;
            long notnullval;
            if (paccessor != null) {
                paccessor.paSetActivation((Serializable) tuple.getObject(3));
                paccessor.paSetActivityPerformer(tuple.getString(4));
                paccessor.paSetAnticipable(tuple.getBoolean(5));
                paccessor.paSetBnHooks(bnHooksPnc.decodeAbstract(getInstancePNGPrefetchbnHooks(), tuple), conn);
                paccessor.paSetBnInterHooks(bnInterHooksPnc.decodeAbstract(getInstancePNGPrefetchbnInterHooks(), tuple), conn);
                paccessor.paSetBnNodePerformerAssign(bnNodePerformerAssignPnc.decodeAbstract(getInstancePNGPrefetchbnNodePerformerAssign(), tuple), conn);
                paccessor.paSetBnProject(bnProjectPnc.decodeAbstract(getInstancePNGPrefetchbnProject(), tuple), conn);
                paccessor.paSetBnProperties(bnPropertiesPnc.decodeAbstract(getInstancePNGPrefetchbnProperties(), tuple), conn);
                paccessor.paSetBnRole(bnRolePnc.decodeAbstract(getInstancePNGPrefetchbnRole(), tuple), conn);
                paccessor.paSetCreationDate(tuple.getDate(6));
                paccessor.paSetCreator(tuple.getString(7));
                paccessor.paSetDeadlines((Serializable) tuple.getObject(8));
                paccessor.paSetDescription(tuple.getString(9));
                paccessor.paSetEndDate(tuple.getDate(10));
                paccessor.paSetExecutor(tuple.getString(11));
                paccessor.paSetId(tuple.getString(12));
                paccessor.paSetInBnAgentEdges(inBnAgentEdgesPnc.decodeAbstract(getInstancePNGPrefetchinBnAgentEdges(), tuple), conn);
                paccessor.paSetInBnEdges(inBnEdgesPnc.decodeAbstract(getInstancePNGPrefetchinBnEdges(), tuple), conn);
                paccessor.paSetName(tuple.getString(14));
                paccessor.paSetOutBnEdges(outBnEdgesPnc.decodeAbstract(getInstancePNGPrefetchoutBnEdges(), tuple), conn);
                paccessor.paSetRelativeDeadlines((Serializable) tuple.getObject(15));
                paccessor.paSetStartDate(tuple.getDate(16));
                paccessor.paSetState(tuple.getInt(17));
                paccessor.paSetTransition((Serializable) tuple.getObject(18));
                paccessor.paSetType(tuple.getInt(19));
            } else if (paccessorGen != null) {
                paccessorGen.paSetSerializedField(activation_NAME, (Serializable) tuple.getObject(3));
                paccessorGen.paSetStringField(activityPerformer_NAME, tuple.getString(4));
                paccessorGen.paSetBooleanField(anticipable_NAME, tuple.getBoolean(5));
                paccessorGen.paSetRefField(bnHooks_NAME, bnHooksPnc.decodeAbstract(getInstancePNGPrefetchbnHooks(), tuple), conn);
                paccessorGen.paSetRefField(bnInterHooks_NAME, bnInterHooksPnc.decodeAbstract(getInstancePNGPrefetchbnInterHooks(), tuple), conn);
                paccessorGen.paSetRefField(bnNodePerformerAssign_NAME, bnNodePerformerAssignPnc.decodeAbstract(getInstancePNGPrefetchbnNodePerformerAssign(), tuple), conn);
                paccessorGen.paSetRefField(bnProject_NAME, bnProjectPnc.decodeAbstract(getInstancePNGPrefetchbnProject(), tuple), conn);
                paccessorGen.paSetRefField(bnProperties_NAME, bnPropertiesPnc.decodeAbstract(getInstancePNGPrefetchbnProperties(), tuple), conn);
                paccessorGen.paSetRefField(bnRole_NAME, bnRolePnc.decodeAbstract(getInstancePNGPrefetchbnRole(), tuple), conn);
                paccessorGen.paSetDateField(creationDate_NAME, tuple.getDate(6));
                paccessorGen.paSetStringField(creator_NAME, tuple.getString(7));
                paccessorGen.paSetSerializedField(deadlines_NAME, (Serializable) tuple.getObject(8));
                paccessorGen.paSetStringField(description_NAME, tuple.getString(9));
                paccessorGen.paSetDateField(endDate_NAME, tuple.getDate(10));
                paccessorGen.paSetStringField(executor_NAME, tuple.getString(11));
                paccessorGen.paSetStringField(id_NAME, tuple.getString(12));
                paccessorGen.paSetRefField(inBnAgentEdges_NAME, inBnAgentEdgesPnc.decodeAbstract(getInstancePNGPrefetchinBnAgentEdges(), tuple), conn);
                paccessorGen.paSetRefField(inBnEdges_NAME, inBnEdgesPnc.decodeAbstract(getInstancePNGPrefetchinBnEdges(), tuple), conn);
                paccessorGen.paSetStringField(name_NAME, tuple.getString(14));
                paccessorGen.paSetRefField(outBnEdges_NAME, outBnEdgesPnc.decodeAbstract(getInstancePNGPrefetchoutBnEdges(), tuple), conn);
                paccessorGen.paSetSerializedField(relativeDeadlines_NAME, (Serializable) tuple.getObject(15));
                paccessorGen.paSetDateField(startDate_NAME, tuple.getDate(16));
                paccessorGen.paSetIntField(state_NAME, tuple.getInt(17));
                paccessorGen.paSetSerializedField(transition_NAME, (Serializable) tuple.getObject(18));
                paccessorGen.paSetIntField(type_NAME, tuple.getInt(19));
            }
			((PBindingCtrl) pb).setStatus(nextState);
		} catch (MedorException me) {
		    throw new PExceptionProtocol(me, "Problem while trying to prefetch data from Medor");
		}
    }

	public boolean exist(PBinding pb, Object conn) throws PException {
		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Exist");
		byte status = pb.getStatus();
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_EXIST);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized EXIST operation, status=" + status);
		((PBindingCtrl) pb).setStatus(nextState);
		PName pname = pb.getPName();
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        PException resex = null;
        boolean exist = false;
		try {
            pstmt = connection.prepareStatement("SELECT BnNode.id FROM BnNode WHERE BnNode.id = ?");
            Object _pngId = pname.encodeAbstract();
            if (_pngId instanceof jorm.BnNodePKPNG) {
                adapter.setString(pstmt, 1, ((jorm.BnNodePKPNG) _pngId).pnGetId(null));
            } else if (_pngId instanceof PNameGetter) {
                adapter.setString(pstmt, 1, ((PNameGetter) _pngId).pngetStringField("id", null));
            } else {
                throw new PExceptionNaming("Invalid abstract encoded value for ID PName: " + _pngId);
            }
			rs = pstmt.executeQuery();
            exist = rs.next();
		} catch (java.sql.SQLException se) {
			resex = new PExceptionIO(se, "SQL problem while testing the existence of a DSI.");
		} finally {
		    try {
			    if (rs != null) {
			        rs.close();
			        rs = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
					resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		return exist;
	}
    private static BnNode_bnHooksPNGPrefetch singletonPNGPrefetchbnHooks = null;

    private BnNode_bnHooksPNGPrefetch getInstancePNGPrefetchbnHooks() throws PException {
        if (singletonPNGPrefetchbnHooks==null) {
            singletonPNGPrefetchbnHooks = new BnNode_bnHooksPNGPrefetch(bnHooksPnc);
		}
		return singletonPNGPrefetchbnHooks;
	}
    private static BnNode_bnInterHooksPNGPrefetch singletonPNGPrefetchbnInterHooks = null;

    private BnNode_bnInterHooksPNGPrefetch getInstancePNGPrefetchbnInterHooks() throws PException {
        if (singletonPNGPrefetchbnInterHooks==null) {
            singletonPNGPrefetchbnInterHooks = new BnNode_bnInterHooksPNGPrefetch(bnInterHooksPnc);
		}
		return singletonPNGPrefetchbnInterHooks;
	}
    private static BnNode_bnNodePerformerAssignPNGPrefetch singletonPNGPrefetchbnNodePerformerAssign = null;

    private BnNode_bnNodePerformerAssignPNGPrefetch getInstancePNGPrefetchbnNodePerformerAssign() throws PException {
        if (singletonPNGPrefetchbnNodePerformerAssign==null) {
            singletonPNGPrefetchbnNodePerformerAssign = new BnNode_bnNodePerformerAssignPNGPrefetch(bnNodePerformerAssignPnc);
		}
		return singletonPNGPrefetchbnNodePerformerAssign;
	}
    private static BnNode_bnProjectPNGPrefetch singletonPNGPrefetchbnProject = null;

    private BnNode_bnProjectPNGPrefetch getInstancePNGPrefetchbnProject() throws PException {
        if (singletonPNGPrefetchbnProject==null) {
            singletonPNGPrefetchbnProject = new BnNode_bnProjectPNGPrefetch(bnProjectPnc);
		}
		return singletonPNGPrefetchbnProject;
	}
    private static BnNode_bnPropertiesPNGPrefetch singletonPNGPrefetchbnProperties = null;

    private BnNode_bnPropertiesPNGPrefetch getInstancePNGPrefetchbnProperties() throws PException {
        if (singletonPNGPrefetchbnProperties==null) {
            singletonPNGPrefetchbnProperties = new BnNode_bnPropertiesPNGPrefetch(bnPropertiesPnc);
		}
		return singletonPNGPrefetchbnProperties;
	}
    private static BnNode_bnRolePNGPrefetch singletonPNGPrefetchbnRole = null;

    private BnNode_bnRolePNGPrefetch getInstancePNGPrefetchbnRole() throws PException {
        if (singletonPNGPrefetchbnRole==null) {
            singletonPNGPrefetchbnRole = new BnNode_bnRolePNGPrefetch(bnRolePnc);
		}
		return singletonPNGPrefetchbnRole;
	}
    private static BnNode_inBnAgentEdgesPNGPrefetch singletonPNGPrefetchinBnAgentEdges = null;

    private BnNode_inBnAgentEdgesPNGPrefetch getInstancePNGPrefetchinBnAgentEdges() throws PException {
        if (singletonPNGPrefetchinBnAgentEdges==null) {
            singletonPNGPrefetchinBnAgentEdges = new BnNode_inBnAgentEdgesPNGPrefetch(inBnAgentEdgesPnc);
		}
		return singletonPNGPrefetchinBnAgentEdges;
	}
    private static BnNode_inBnEdgesPNGPrefetch singletonPNGPrefetchinBnEdges = null;

    private BnNode_inBnEdgesPNGPrefetch getInstancePNGPrefetchinBnEdges() throws PException {
        if (singletonPNGPrefetchinBnEdges==null) {
            singletonPNGPrefetchinBnEdges = new BnNode_inBnEdgesPNGPrefetch(inBnEdgesPnc);
		}
		return singletonPNGPrefetchinBnEdges;
	}
    private static BnNode_outBnEdgesPNGPrefetch singletonPNGPrefetchoutBnEdges = null;

    private BnNode_outBnEdgesPNGPrefetch getInstancePNGPrefetchoutBnEdges() throws PException {
        if (singletonPNGPrefetchoutBnEdges==null) {
            singletonPNGPrefetchoutBnEdges = new BnNode_outBnEdgesPNGPrefetch(outBnEdgesPnc);
		}
		return singletonPNGPrefetchoutBnEdges;
	}

	//the map of [pclassmapping->table of int]
	//build dynamically
	private HashMap associationTable =  new HashMap();
	
	public HashMap getAssociationTable(){
		return associationTable;
	}
	
	public void addAssociation(PClassMapping targetClass, int[] indexes){
		associationTable.put(targetClass, indexes);
	}
	
	public int[] getIndexesTable(PClassMapping targetClass){
		return (int[]) associationTable.get(targetClass);
	}
	

    private static jorm.BnNodePerformerAssignPKPNG singletonPNG9 = null;
    public jorm.BnNodePerformerAssignPKPNG getInstancePNG9() throws PException {
        if (singletonPNG9==null) {
            singletonPNG9 = new  BnNodePNG9(bnNodePerformerAssignPnc, adapter);
		}
		return singletonPNG9;
	}

    private static jorm.BnNodePKPNG singletonPNG1 = null;
    public jorm.BnNodePKPNG getInstancePNG1() throws PException {
        if (singletonPNG1==null) {
            singletonPNG1 = new  BnNodePNG1(bnInterHooksPnc, adapter);
		}
		return singletonPNG1;
	}

    private static jorm.BnNodePKPNG singletonPNG4 = null;
    public jorm.BnNodePKPNG getInstancePNG4() throws PException {
        if (singletonPNG4==null) {
            singletonPNG4 = new  BnNodePNG4(bnPropertiesPnc, adapter);
		}
		return singletonPNG4;
	}

    private static jorm.BnNodePKPNG singletonPNG8 = null;
    public jorm.BnNodePKPNG getInstancePNG8() throws PException {
        if (singletonPNG8==null) {
            singletonPNG8 = new  BnNodePNG8(inBnEdgesPnc, adapter);
		}
		return singletonPNG8;
	}

    private static jorm.BnProjectPKPNG singletonPNG5 = null;
    public jorm.BnProjectPKPNG getInstancePNG5() throws PException {
        if (singletonPNG5==null) {
            singletonPNG5 = new  BnNodePNG5(bnProjectPnc, adapter);
		}
		return singletonPNG5;
	}

    private static jorm.BnNodePKPNG singletonPNG3 = null;
    public jorm.BnNodePKPNG getInstancePNG3() throws PException {
        if (singletonPNG3==null) {
            singletonPNG3 = new  BnNodePNG3(outBnEdgesPnc, adapter);
		}
		return singletonPNG3;
	}

    private static jorm.BnNodePKPNG singletonPNG7 = null;
    public jorm.BnNodePKPNG getInstancePNG7() throws PException {
        if (singletonPNG7==null) {
            singletonPNG7 = new  BnNodePNG7(bnHooksPnc, adapter);
		}
		return singletonPNG7;
	}

    private static jorm.BnNodePKPNG singletonPNG6 = null;
    public jorm.BnNodePKPNG getInstancePNG6() throws PException {
        if (singletonPNG6==null) {
            singletonPNG6 = new  BnNodePNG6(inBnAgentEdgesPnc, adapter);
		}
		return singletonPNG6;
	}

    private static jorm.BnRolePKPNG singletonPNG2 = null;
    public jorm.BnRolePKPNG getInstancePNG2() throws PException {
        if (singletonPNG2==null) {
            singletonPNG2 = new  BnNodePNG2(bnRolePnc, adapter);
		}
		return singletonPNG2;
	}

    private String updateBnNodeTable(
        Connection connection,
        Object conn,
        PAccessorGen paccessorGen,
        BnNodeAccessor paccessor,
    	PName pname)
        throws PException, SQLException, IOException, RdbAdapterException {
		java.sql.Blob blob = null;
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            jorm.BnNodePKPNG _pngId =
                (jorm.BnNodePKPNG)
                pname.encodeAbstract();
            if (paccessor != null) {
                jorm.BnNodePKPNG png1 =
                    (jorm.BnNodePKPNG)
                    bnInterHooksPnc.encodeAbstract(
                        paccessor.paGetBnInterHooks(conn));
                jorm.BnNodePKPNG png4 =
                    (jorm.BnNodePKPNG)
                    bnPropertiesPnc.encodeAbstract(
                        paccessor.paGetBnProperties(conn));
                jorm.BnNodePKPNG png8 =
                    (jorm.BnNodePKPNG)
                    inBnEdgesPnc.encodeAbstract(
                        paccessor.paGetInBnEdges(conn));
                jorm.BnProjectPKPNG png5 =
                    (jorm.BnProjectPKPNG)
                    bnProjectPnc.encodeAbstract(
                        paccessor.paGetBnProject(conn));
                jorm.BnNodePKPNG png3 =
                    (jorm.BnNodePKPNG)
                    outBnEdgesPnc.encodeAbstract(
                        paccessor.paGetOutBnEdges(conn));
                jorm.BnNodePKPNG png7 =
                    (jorm.BnNodePKPNG)
                    bnHooksPnc.encodeAbstract(
                        paccessor.paGetBnHooks(conn));
                jorm.BnNodePKPNG png6 =
                    (jorm.BnNodePKPNG)
                    inBnAgentEdgesPnc.encodeAbstract(
                        paccessor.paGetInBnAgentEdges(conn));
                jorm.BnRolePKPNG png2 =
                    (jorm.BnRolePKPNG)
                    bnRolePnc.encodeAbstract(
                        paccessor.paGetBnRole(conn));
            pstmt = connection.prepareStatement(res = "UPDATE BnNode SET startdate = ?, executor = ?, anticipable = ?, enddate = ?, type = ?, transition = ?, activityPerformer = ?, deadlines = ?, relativedeadlines = ?, BnProject_id = ?, id = ?, BnRole_id = ?, creator = ?, description = ?, state = ?, activation = ?, name = ?, creationdate = ? WHERE id = ?");
				//startdate
                nullField = paccessor.paGetStartDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 1, ((java.util.Date) nullField), null);
                }
				//executor
                nullField = paccessor.paGetExecutor();
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//anticipable
				adapter.setBoolean(pstmt, 3, paccessor.paGetAnticipable());
				//enddate
                nullField = paccessor.paGetEndDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 4, ((java.util.Date) nullField), null);
                }
				//type
				adapter.setInt(pstmt, 5, paccessor.paGetType());
				//transition
                nullField = paccessor.paGetTransition();
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 6, ((java.io.Serializable) nullField));
                }
				//activityPerformer
                nullField = paccessor.paGetActivityPerformer();
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 7, ((java.lang.String) nullField));
                }
				//deadlines
                nullField = paccessor.paGetDeadlines();
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 8, ((java.io.Serializable) nullField));
                }
				//relativedeadlines
                nullField = paccessor.paGetRelativeDeadlines();
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 9, ((java.io.Serializable) nullField));
                }
				//BnProject_id
                nullField = png5.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 10, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 10, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 11, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 11, ((java.lang.String) nullField));
                }
				//BnRole_id
                nullField = png2.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 12, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 12, ((java.lang.String) nullField));
                }
				//creator
                nullField = paccessor.paGetCreator();
                if (nullField == null) {
                    adapter.setNull(pstmt, 13, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 13, ((java.lang.String) nullField));
                }
				//description
                nullField = paccessor.paGetDescription();
                if (nullField == null) {
                    adapter.setNull(pstmt, 14, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 14, ((java.lang.String) nullField));
                }
				//state
				adapter.setInt(pstmt, 15, paccessor.paGetState());
				//activation
                nullField = paccessor.paGetActivation();
                if (nullField == null) {
                    adapter.setNull(pstmt, 16, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 16, ((java.io.Serializable) nullField));
                }
				//name
                nullField = paccessor.paGetName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 17, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 17, ((java.lang.String) nullField));
                }
				//creationdate
                nullField = paccessor.paGetCreationDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 18, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 18, ((java.util.Date) nullField), null);
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 19, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 19, ((java.lang.String) nullField));
                }
            } else if (paccessorGen != null) {
                jorm.BnNodePKPNG png1 =
                    (jorm.BnNodePKPNG)
                    bnInterHooksPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnInterHooks_NAME, conn));
                jorm.BnNodePKPNG png4 =
                    (jorm.BnNodePKPNG)
                    bnPropertiesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnProperties_NAME, conn));
                jorm.BnNodePKPNG png8 =
                    (jorm.BnNodePKPNG)
                    inBnEdgesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(inBnEdges_NAME, conn));
                jorm.BnProjectPKPNG png5 =
                    (jorm.BnProjectPKPNG)
                    bnProjectPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnProject_NAME, conn));
                jorm.BnNodePKPNG png3 =
                    (jorm.BnNodePKPNG)
                    outBnEdgesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(outBnEdges_NAME, conn));
                jorm.BnNodePKPNG png7 =
                    (jorm.BnNodePKPNG)
                    bnHooksPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnHooks_NAME, conn));
                jorm.BnNodePKPNG png6 =
                    (jorm.BnNodePKPNG)
                    inBnAgentEdgesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(inBnAgentEdges_NAME, conn));
                jorm.BnRolePKPNG png2 =
                    (jorm.BnRolePKPNG)
                    bnRolePnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnRole_NAME, conn));
            pstmt = connection.prepareStatement(res = "UPDATE BnNode SET startdate = ?, executor = ?, anticipable = ?, enddate = ?, type = ?, transition = ?, activityPerformer = ?, deadlines = ?, relativedeadlines = ?, BnProject_id = ?, id = ?, BnRole_id = ?, creator = ?, description = ?, state = ?, activation = ?, name = ?, creationdate = ? WHERE id = ?");
				//startdate
                nullField = paccessorGen.paGetDateField(startDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 1, ((java.util.Date) nullField), null);
                }
				//executor
                nullField = paccessorGen.paGetStringField(executor_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//anticipable
				adapter.setBoolean(pstmt, 3, paccessorGen.paGetBooleanField(anticipable_NAME));
				//enddate
                nullField = paccessorGen.paGetDateField(endDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 4, ((java.util.Date) nullField), null);
                }
				//type
				adapter.setInt(pstmt, 5, paccessorGen.paGetIntField(type_NAME));
				//transition
                nullField = paccessorGen.paGetSerializedField(transition_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 6, ((java.io.Serializable) nullField));
                }
				//activityPerformer
                nullField = paccessorGen.paGetStringField(activityPerformer_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 7, ((java.lang.String) nullField));
                }
				//deadlines
                nullField = paccessorGen.paGetSerializedField(deadlines_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 8, ((java.io.Serializable) nullField));
                }
				//relativedeadlines
                nullField = paccessorGen.paGetSerializedField(relativeDeadlines_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 9, ((java.io.Serializable) nullField));
                }
				//BnProject_id
                nullField = png5.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 10, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 10, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessorGen.paGetStringField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 11, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 11, ((java.lang.String) nullField));
                }
				//BnRole_id
                nullField = png2.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 12, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 12, ((java.lang.String) nullField));
                }
				//creator
                nullField = paccessorGen.paGetStringField(creator_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 13, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 13, ((java.lang.String) nullField));
                }
				//description
                nullField = paccessorGen.paGetStringField(description_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 14, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 14, ((java.lang.String) nullField));
                }
				//state
				adapter.setInt(pstmt, 15, paccessorGen.paGetIntField(state_NAME));
				//activation
                nullField = paccessorGen.paGetSerializedField(activation_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 16, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 16, ((java.io.Serializable) nullField));
                }
				//name
                nullField = paccessorGen.paGetStringField(name_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 17, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 17, ((java.lang.String) nullField));
                }
				//creationdate
                nullField = paccessorGen.paGetDateField(creationDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 18, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 18, ((java.util.Date) nullField), null);
                }
				//id
                nullField = paccessorGen.paGetStringField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 19, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 19, ((java.lang.String) nullField));
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String insertBnNodeTable(Connection connection,
                                                Object conn,
                                                PAccessorGen paccessorGen,
                                                BnNodeAccessor paccessor,
    											PName pname)
        throws PException, SQLException, IOException, RdbAdapterException {

		Object nullField = null;
		java.sql.Blob blob = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            jorm.BnNodePKPNG _pngId =
                (jorm.BnNodePKPNG)
                pname.encodeAbstract();
            if (paccessor != null) {
                jorm.BnNodePKPNG png1 =
                    (jorm.BnNodePKPNG)
                    bnInterHooksPnc.encodeAbstract(
                        paccessor.paGetBnInterHooks(conn));
                jorm.BnNodePKPNG png4 =
                    (jorm.BnNodePKPNG)
                    bnPropertiesPnc.encodeAbstract(
                        paccessor.paGetBnProperties(conn));
                jorm.BnNodePKPNG png8 =
                    (jorm.BnNodePKPNG)
                    inBnEdgesPnc.encodeAbstract(
                        paccessor.paGetInBnEdges(conn));
                jorm.BnProjectPKPNG png5 =
                    (jorm.BnProjectPKPNG)
                    bnProjectPnc.encodeAbstract(
                        paccessor.paGetBnProject(conn));
                jorm.BnNodePKPNG png3 =
                    (jorm.BnNodePKPNG)
                    outBnEdgesPnc.encodeAbstract(
                        paccessor.paGetOutBnEdges(conn));
                jorm.BnNodePKPNG png7 =
                    (jorm.BnNodePKPNG)
                    bnHooksPnc.encodeAbstract(
                        paccessor.paGetBnHooks(conn));
                jorm.BnNodePKPNG png6 =
                    (jorm.BnNodePKPNG)
                    inBnAgentEdgesPnc.encodeAbstract(
                        paccessor.paGetInBnAgentEdges(conn));
                jorm.BnRolePKPNG png2 =
                    (jorm.BnRolePKPNG)
                    bnRolePnc.encodeAbstract(
                        paccessor.paGetBnRole(conn));
                pstmt = connection.prepareStatement(res = "INSERT INTO BnNode (startdate, executor, anticipable, enddate, type, transition, activityPerformer, deadlines, relativedeadlines, BnProject_id, id, BnRole_id, creator, description, state, activation, name, creationdate) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
				//startdate
                nullField = paccessor.paGetStartDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 1, ((java.util.Date) nullField), null);
                }
				//executor
                nullField = paccessor.paGetExecutor();
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//anticipable
				adapter.setBoolean(pstmt, 3, paccessor.paGetAnticipable());
				//enddate
                nullField = paccessor.paGetEndDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 4, ((java.util.Date) nullField), null);
                }
				//type
				adapter.setInt(pstmt, 5, paccessor.paGetType());
				//transition
                nullField = paccessor.paGetTransition();
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 6, ((java.io.Serializable) nullField));
                }
				//activityPerformer
                nullField = paccessor.paGetActivityPerformer();
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 7, ((java.lang.String) nullField));
                }
				//deadlines
                nullField = paccessor.paGetDeadlines();
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 8, ((java.io.Serializable) nullField));
                }
				//relativedeadlines
                nullField = paccessor.paGetRelativeDeadlines();
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 9, ((java.io.Serializable) nullField));
                }
				//BnProject_id
                nullField = png5.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 10, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 10, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 11, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 11, ((java.lang.String) nullField));
                }
				//BnRole_id
                nullField = png2.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 12, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 12, ((java.lang.String) nullField));
                }
				//creator
                nullField = paccessor.paGetCreator();
                if (nullField == null) {
                    adapter.setNull(pstmt, 13, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 13, ((java.lang.String) nullField));
                }
				//description
                nullField = paccessor.paGetDescription();
                if (nullField == null) {
                    adapter.setNull(pstmt, 14, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 14, ((java.lang.String) nullField));
                }
				//state
				adapter.setInt(pstmt, 15, paccessor.paGetState());
				//activation
                nullField = paccessor.paGetActivation();
                if (nullField == null) {
                    adapter.setNull(pstmt, 16, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 16, ((java.io.Serializable) nullField));
                }
				//name
                nullField = paccessor.paGetName();
                if (nullField == null) {
                    adapter.setNull(pstmt, 17, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 17, ((java.lang.String) nullField));
                }
				//creationdate
                nullField = paccessor.paGetCreationDate();
                if (nullField == null) {
                    adapter.setNull(pstmt, 18, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 18, ((java.util.Date) nullField), null);
                }
            } else if (paccessorGen != null) {
                jorm.BnNodePKPNG png1 =
                    (jorm.BnNodePKPNG)
                    bnInterHooksPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnInterHooks_NAME, conn));
                jorm.BnNodePKPNG png4 =
                    (jorm.BnNodePKPNG)
                    bnPropertiesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnProperties_NAME, conn));
                jorm.BnNodePKPNG png8 =
                    (jorm.BnNodePKPNG)
                    inBnEdgesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(inBnEdges_NAME, conn));
                jorm.BnProjectPKPNG png5 =
                    (jorm.BnProjectPKPNG)
                    bnProjectPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnProject_NAME, conn));
                jorm.BnNodePKPNG png3 =
                    (jorm.BnNodePKPNG)
                    outBnEdgesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(outBnEdges_NAME, conn));
                jorm.BnNodePKPNG png7 =
                    (jorm.BnNodePKPNG)
                    bnHooksPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnHooks_NAME, conn));
                jorm.BnNodePKPNG png6 =
                    (jorm.BnNodePKPNG)
                    inBnAgentEdgesPnc.encodeAbstract(
                        paccessorGen.paGetRefField(inBnAgentEdges_NAME, conn));
                jorm.BnRolePKPNG png2 =
                    (jorm.BnRolePKPNG)
                    bnRolePnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnRole_NAME, conn));
                pstmt = connection.prepareStatement(res = "INSERT INTO BnNode (startdate, executor, anticipable, enddate, type, transition, activityPerformer, deadlines, relativedeadlines, BnProject_id, id, BnRole_id, creator, description, state, activation, name, creationdate) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
				//startdate
                nullField = paccessorGen.paGetDateField(startDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 1, ((java.util.Date) nullField), null);
                }
				//executor
                nullField = paccessorGen.paGetStringField(executor_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//anticipable
				adapter.setBoolean(pstmt, 3, paccessorGen.paGetBooleanField(anticipable_NAME));
				//enddate
                nullField = paccessorGen.paGetDateField(endDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 4, ((java.util.Date) nullField), null);
                }
				//type
				adapter.setInt(pstmt, 5, paccessorGen.paGetIntField(type_NAME));
				//transition
                nullField = paccessorGen.paGetSerializedField(transition_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 6, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 6, ((java.io.Serializable) nullField));
                }
				//activityPerformer
                nullField = paccessorGen.paGetStringField(activityPerformer_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 7, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 7, ((java.lang.String) nullField));
                }
				//deadlines
                nullField = paccessorGen.paGetSerializedField(deadlines_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 8, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 8, ((java.io.Serializable) nullField));
                }
				//relativedeadlines
                nullField = paccessorGen.paGetSerializedField(relativeDeadlines_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 9, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 9, ((java.io.Serializable) nullField));
                }
				//BnProject_id
                nullField = png5.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 10, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 10, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessorGen.paGetStringField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 11, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 11, ((java.lang.String) nullField));
                }
				//BnRole_id
                nullField = png2.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 12, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 12, ((java.lang.String) nullField));
                }
				//creator
                nullField = paccessorGen.paGetStringField(creator_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 13, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 13, ((java.lang.String) nullField));
                }
				//description
                nullField = paccessorGen.paGetStringField(description_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 14, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 14, ((java.lang.String) nullField));
                }
				//state
				adapter.setInt(pstmt, 15, paccessorGen.paGetIntField(state_NAME));
				//activation
                nullField = paccessorGen.paGetSerializedField(activation_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 16, adapter.getSqlTypeCode(PTypeSpace.SERIALIZED.getTypeCode(), null));
                } else {
                   	adapter.setSerialized(pstmt, 16, ((java.io.Serializable) nullField));
                }
				//name
                nullField = paccessorGen.paGetStringField(name_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 17, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 17, ((java.lang.String) nullField));
                }
				//creationdate
                nullField = paccessorGen.paGetDateField(creationDate_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 18, adapter.getSqlTypeCode(PTypeSpace.DATE.getTypeCode(), null));
                } else {
                   	adapter.setDate(pstmt, 18, ((java.util.Date) nullField), null);
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String deleteBnNodeTable(
    	Connection connection,
    	Object conn,
    	PAccessorGen paccessorGen,
    	BnNodeAccessor paccessor,
    	PName pname) throws PException, SQLException, RdbAdapterException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            pstmt = connection.prepareStatement(res = "DELETE FROM BnNode WHERE id = ?");
        jorm.BnNodePKPNG _pngId =
            (jorm.BnNodePKPNG)
            pname.encodeAbstract();
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }

	/**
	 * Is the RdbAdapter to use for managing the various relational database.
	 * This field must not be null after the mapping of the class
	 */
	public RdbAdapter adapter;
	/**
	 * Is the select query to use in the Binding.read method
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The read query is built once time
	 * when the class is mapped.
	 */
	public String readQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String prefetchedExtentQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is NOT activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String extentQuery = null;

	/**
	 * Building of the structure defining the FROM clause with
	 *  JoinedTable instances
	 */
	private JoinedTable getJoinedTable() {
		JoinedTable jt = new JoinedTable("BnNode");
		JoinedTable.Join jtj = null;
		jtj = jt.createChildren("BnNodePerformerAssign");
		jtj.addJoinColumn("id", "BnNode_id");
        return jt;
	}

    public String[] getPrefetchSelectCols() {
		return new String[]{"BnNode.BnProject_id", "BnNode.BnRole_id", "BnNode.activation", "BnNode.activityPerformer", "BnNode.anticipable", "BnNode.creationdate", "BnNode.creator", "BnNode.deadlines", "BnNode.description", "BnNode.enddate", "BnNode.executor", "BnNode.id", "BnNodePerformerAssign.id", "BnNode.name", "BnNode.relativedeadlines", "BnNode.startdate", "BnNode.state", "BnNode.transition", "BnNode.type"};
	}

	public String getPrefetchWhereClause() {
		return null;
	}

	public java.util.Properties getCNId2Column() {
		java.util.Properties p = new java.util.Properties();
		p.setProperty("id", "BnNode.id");
		return p;
	}

	public List getPrefetchTables() {
		return Collections.singletonList(getJoinedTable());
	}

	/**
	 * Compute the read query with the help of the RdbAdapter
	 */
	public void computeReadQuery() {
		readQuery = adapter.getQuery("BnNode.BnProject_id, BnNode.BnRole_id, BnNode.activation, BnNode.activityPerformer, BnNode.anticipable, BnNode.creationdate, BnNode.creator, BnNode.deadlines, BnNode.description, BnNode.enddate, BnNode.executor, BnNode.id, BnNodePerformerAssign.id, BnNode.name, BnNode.relativedeadlines, BnNode.startdate, BnNode.state, BnNode.transition, BnNode.type",
			Collections.singletonList(getJoinedTable()),
			"BnNode.id = ?",
			false, false);
	}

	/**
	 * Compute the extent query (without prefetch) with the help of the
	 * RdbAdapter
	 */
	public void computeExtentQuery() throws ExpressionException {
		if (extentQuery == null) {
			extentQuery = adapter.getQuery("BnNode.id",
				Collections.singletonList(getJoinedTable()),
			null,
			false, false);
		}
	}

	/**
	 * Compute the extent query (with prefetch) with the help of the RdbAdapter
	 */
	public void computePrefetchedExtentQuery() {
		if (prefetchedExtentQuery == null) {
			prefetchedExtentQuery = adapter.getQuery("BnNode.BnProject_id, BnNode.BnRole_id, BnNode.activation, BnNode.activityPerformer, BnNode.anticipable, BnNode.creationdate, BnNode.creator, BnNode.deadlines, BnNode.description, BnNode.enddate, BnNode.executor, BnNode.id, BnNodePerformerAssign.id, BnNode.name, BnNode.relativedeadlines, BnNode.startdate, BnNode.state, BnNode.transition, BnNode.type, BnNode.id" + adapter.getColumnAliasExpr("pk0") + "",
			Collections.singletonList(getJoinedTable()),
			null,
			false, false);
		}

	}


	public PName decode(ResultSet rs, int startindex, Object png) throws IOException, SQLException, PException {
        return classPNC.decodeAbstract(png, null);
	}



	
    // IMPLEMENTATION OF THE SPECIFIC PNAME GETTER
    public java.lang.String pnGetId(Object ctx)
            throws PException {
	    BnNodeAccessor paccessor = null;
		if (!(ctx instanceof BnNodeAccessor)) {
			throw new PExceptionProtocol("Unsupported context type: " + ctx);
		}
        try {
            return ((BnNodeAccessor) ctx).paGetId();
        } catch (Exception e) {
		    throw new PException(e, "Cannot read the field 'id'");
        }
    }
    // IMPLEMENTATION OF THE PNameGetter INTERFACE

    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
		if ("id".equals(fn)) {
			if (ctx instanceof BnNodeAccessor) {
				return ((BnNodeAccessor) ctx).paGetId();
			} else {
			    throw new PExceptionProtocol("Unsupported context type: " + ctx);
			}
        }
        throw new PException("Cannot read the field " + fn);
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	public void setLogger(Logger l) {
		logger = l;
		((Loggable) bnHooksGCM).setLogger(l);
		((Loggable) bnInterHooksGCM).setLogger(l);
		((Loggable) bnPropertiesGCM).setLogger(l);
		((Loggable) inBnAgentEdgesGCM).setLogger(l);
		((Loggable) inBnEdgesGCM).setLogger(l);
		((Loggable) outBnEdgesGCM).setLogger(l);
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}
}


/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnNodePNG9
    implements PNameGetter, jorm.BnNodePerformerAssignPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnNodePNG9(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnNodePerformerAssignPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 13, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 13, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnNodePNG1
    implements PNameGetter, jorm.BnNodePKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnNodePNG1(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnNodePKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 12, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 12, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnNodePNG4
    implements PNameGetter, jorm.BnNodePKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnNodePNG4(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnNodePKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 12, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 12, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnNodePNG8
    implements PNameGetter, jorm.BnNodePKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnNodePNG8(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnNodePKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 12, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 12, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnNodePNG5
    implements PNameGetter, jorm.BnProjectPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnNodePNG5(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 1, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 1, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnNodePNG3
    implements PNameGetter, jorm.BnNodePKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnNodePNG3(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnNodePKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 12, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 12, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnNodePNG7
    implements PNameGetter, jorm.BnNodePKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnNodePNG7(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnNodePKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 12, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 12, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnNodePNG6
    implements PNameGetter, jorm.BnNodePKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnNodePNG6(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnNodePKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 12, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 12, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnNodePNG2
    implements PNameGetter, jorm.BnRolePKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnNodePNG2(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnRolePKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 2, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 2, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnNode_bnHooksPNGPrefetch
        implements PNameGetter, jorm.BnNodePKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnNode_bnHooksPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnNodePKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(12)) {
                    return ((Tuple) ctx).getString(12);
                } else {
                    return ((jorm.BnNodePKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(12)) {
                        return ((Tuple) ctx).getString(12);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnNode_bnInterHooksPNGPrefetch
        implements PNameGetter, jorm.BnNodePKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnNode_bnInterHooksPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnNodePKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(12)) {
                    return ((Tuple) ctx).getString(12);
                } else {
                    return ((jorm.BnNodePKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(12)) {
                        return ((Tuple) ctx).getString(12);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnNode_bnNodePerformerAssignPNGPrefetch
        implements PNameGetter, jorm.BnNodePerformerAssignPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnNode_bnNodePerformerAssignPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnNodePerformerAssignPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(13)) {
                    return ((Tuple) ctx).getString(13);
                } else {
                    return ((jorm.BnNodePerformerAssignPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(13)) {
                        return ((Tuple) ctx).getString(13);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnNode_bnProjectPNGPrefetch
        implements PNameGetter, jorm.BnProjectPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnNode_bnProjectPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnProjectPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(1)) {
                    return ((Tuple) ctx).getString(1);
                } else {
                    return ((jorm.BnProjectPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(1)) {
                        return ((Tuple) ctx).getString(1);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnNode_bnPropertiesPNGPrefetch
        implements PNameGetter, jorm.BnNodePKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnNode_bnPropertiesPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnNodePKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(12)) {
                    return ((Tuple) ctx).getString(12);
                } else {
                    return ((jorm.BnNodePKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(12)) {
                        return ((Tuple) ctx).getString(12);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnNode_bnRolePNGPrefetch
        implements PNameGetter, jorm.BnRolePKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnNode_bnRolePNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnRolePKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(2)) {
                    return ((Tuple) ctx).getString(2);
                } else {
                    return ((jorm.BnRolePKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(2)) {
                        return ((Tuple) ctx).getString(2);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnNode_inBnAgentEdgesPNGPrefetch
        implements PNameGetter, jorm.BnNodePKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnNode_inBnAgentEdgesPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnNodePKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(12)) {
                    return ((Tuple) ctx).getString(12);
                } else {
                    return ((jorm.BnNodePKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(12)) {
                        return ((Tuple) ctx).getString(12);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnNode_inBnEdgesPNGPrefetch
        implements PNameGetter, jorm.BnNodePKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnNode_inBnEdgesPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnNodePKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(12)) {
                    return ((Tuple) ctx).getString(12);
                } else {
                    return ((jorm.BnNodePKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(12)) {
                        return ((Tuple) ctx).getString(12);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnNode_outBnEdgesPNGPrefetch
        implements PNameGetter, jorm.BnNodePKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnNode_outBnEdgesPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnNodePKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(12)) {
                    return ((Tuple) ctx).getString(12);
                } else {
                    return ((jorm.BnNodePKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(12)) {
                        return ((Tuple) ctx).getString(12);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }



class BnNodeResultIterator implements PNameIterator, jorm.BnNodePKPNG {
    ResultSet rs;
	private PBinder binder;
	private BnNodeMapping pclassMapping;
    private boolean hasNext;
    private boolean isForw;
	private PreparedStatement ps;
	private Logger logger;

    private boolean prefetching;
    
    private PPrefetchTupleCollection tc = null;
    private int pnameIndex;

	BnNodeResultIterator(Object txctx,
		PreparedStatement ps,
		ResultSet r,
		PBinder binder,
		boolean prefetching,
		boolean withSubType,
		Logger l,
		int pi,
		BnNodeMapping pcm) throws SQLException, MedorException {

		this.ps = ps;
		this.binder = binder;
		this.prefetching = prefetching;
        rs = r;
        logger = l;
        pclassMapping = pcm;
        if (!prefetching) {
        	pnameIndex = 1;
        } else {
        	pnameIndex = pi;
        	tc = new RdbTupleCollection(txctx, r, ps, pnameIndex, logger, pclassMapping, withSubType, this);
        }
        hasNext = (!prefetching) ? rs.next() : tc.next();
        isForw = true;
    }

	public void finalize() throws Throwable {
		if (rs != null)
			rs.close();
		if (ps != null)
			ps.close();
		super.finalize();
	}

    public void close() {
        if (!prefetching) {
            if (rs!= null) {
                try {
                    rs.close();
                } catch(Exception e) {
                }
                rs = null;
            }
            hasNext = false;
            isForw = false;
        }
    }

    public boolean hasNext() {
    	boolean res;
        if (isForw) {
            res = hasNext;
        } else {
            isForw = true;
            try {
                hasNext = (!prefetching) ? rs.next() : tc.next();
                res = hasNext;
            } catch(Exception e) {
                e.printStackTrace();
                res = false;
            }
        }
        if (res == false) {
        	try {
				if (rs != null) {
					rs.close();
					rs = null;
				}
				if (ps != null) {
					ps.close();
					ps = null;
				}
			} catch (SQLException e) {
				rs = null;
				ps = null;
			}
        }
        return res;
    }

    public void remove() throws UnsupportedOperationException, IllegalStateException {
        throw new UnsupportedOperationException();
    }

    public Object next() throws NoSuchElementException {
        try {
            if (!isForw) {
                hasNext = (!prefetching) ? rs.next() : tc.next();
                isForw = true;
            }
            if (!hasNext) {
                throw new NoSuchElementException();
            }
            isForw = false;
            if (prefetching) {
            	tc.getPrefetchBuffer().addPrefetchTuple();
            }
            return pclassMapping.decode(rs, -1, this);
        } catch (NoSuchElementException e) {
        	throw e;
        } catch (Exception e) {
        	if (logger != null) {
        		logger.log(BasicLevel.ERROR, "Impossible to fetch the next element", e);
        	}
            throw new NoSuchElementException(
                    "Impossible to decode to an ID PName: " + e.getMessage());
        }
    }

    // IMPLEMENTATION OF THE BnNodePKPNG INTERFACE //

    public java.lang.String pnGetId(Object ctx) throws PExceptionIO, PExceptionTyping {
        try {
            return pclassMapping.adapter.getString(rs, pnameIndex + 0, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Impossible to fetch a the field id of the pname");
        }
    }
}
